# coding: utf-8

import datetime
import unittest

import fastobo



class TestLiteralPropertyValue(unittest.TestCase):

    type = fastobo.pv.LiteralPropertyValue

    def test_init(self):
        rel = fastobo.id.UnprefixedIdent("creation_date")
        value = "2019-04-08T23:21:05Z"
        dt = fastobo.id.PrefixedIdent("xsd", "date")
        try:
            pv = self.type(rel, value, dt)
        except Exception:
            self.fail("could not create `LiteralPropertyValue` instance")

    def test_init_type_error(self):
        rel = fastobo.id.UnprefixedIdent("creation_date")
        value = "2019-04-08T23:21:05Z"
        dt = fastobo.id.PrefixedIdent("xsd", "date")
        self.assertRaises(TypeError, self.type, 1, value, dt)
        self.assertRaises(TypeError, self.type, rel, 1, dt)
        self.assertRaises(TypeError, self.type, rel, value, 1)

    def test_property_relation(self):
        rel = fastobo.id.UnprefixedIdent("creation_date")
        value = "2019-04-08T23:21:05Z"
        dt = fastobo.id.PrefixedIdent("xsd", "date")
        pv = self.type(rel, value, dt)
        self.assertEqual(pv.relation, rel)

        rel2 = fastobo.id.PrefixedIdent("IAO", "0000219")
        pv.relation = rel2
        self.assertEqual(pv.relation, rel2)

        with self.assertRaises(TypeError):
            pv.relation = "IAO:0000219"

    def test_str(self):
        rel = fastobo.id.UnprefixedIdent("creation_date")
        value = "2019-04-08T23:21:05Z"
        dt = fastobo.id.PrefixedIdent("xsd", "date")
        pv = self.type(rel, value, dt)
        self.assertEqual(
            str(pv),
            'creation_date "2019-04-08T23:21:05Z" xsd:date'
        )

    def test_repr(self):
        rel = fastobo.id.UnprefixedIdent("creation_date")
        value = "2019-04-08T23:21:05Z"
        dt = fastobo.id.PrefixedIdent("xsd", "date")
        pv = self.type(rel, value, dt)
        self.assertEqual(
            repr(pv),
            "LiteralPropertyValue("
            "UnprefixedIdent('creation_date'), "
            "'2019-04-08T23:21:05Z', "
            "PrefixedIdent('xsd', 'date'))"
        )



class TestResourcePropertyValue(unittest.TestCase):

    type = fastobo.pv.ResourcePropertyValue

    def test_init(self):
        rel = fastobo.id.UnprefixedIdent("derived_from")
        value = fastobo.id.PrefixedIdent("MS", "1000031")
        try:
            pv = self.type(rel, value)
        except Exception:
            self.fail("could not create `ResourcePropertyValue` instance")

    def test_init_type_error(self):
        rel = fastobo.id.UnprefixedIdent("derived_from")
        value = fastobo.id.PrefixedIdent("MS", "1000031")
        self.assertRaises(TypeError, self.type, 1, value)
        self.assertRaises(TypeError, self.type, rel, 1)

    def test_property_relation(self):
        rel = fastobo.id.UnprefixedIdent("derived_from")
        value = fastobo.id.PrefixedIdent("MS", "1000031")
        pv = self.type(rel, value)
        self.assertEqual(pv.relation, rel)

        rel2 = fastobo.id.UnprefixedIdent("something")
        pv.relation = rel2
        self.assertEqual(pv.relation, rel2)

        with self.assertRaises(TypeError):
            pv.relation = "IAO:0000219"

    def test_str(self):
        rel = fastobo.id.UnprefixedIdent("derived_from")
        value = fastobo.id.PrefixedIdent("MS", "1000031")
        pv = self.type(rel, value)
        self.assertEqual(str(pv), "derived_from MS:1000031")

    def test_repr(self):
        rel = fastobo.id.UnprefixedIdent("derived_from")
        value = fastobo.id.PrefixedIdent("MS", "1000031")
        pv = self.type(rel, value)
        self.assertEqual(
            repr(pv),
            "ResourcePropertyValue("
            "UnprefixedIdent('derived_from'), "
            "PrefixedIdent('MS', '1000031'))"
        )



# coding: utf-8
"""Test doctest contained tests in every file of the module.
"""

import os
import sys
import datetime
import doctest
import warnings
import pprint
import types

import fastobo


def _load_tests_from_module(tests, module, globs, setUp=None, tearDown=None):
    """Load tests from module, iterating through submodules"""

    module.__test__ = {}
    for attr in (getattr(module, x) for x in dir(module) if not x.startswith('_')):
        if isinstance(attr, types.ModuleType):
            _load_tests_from_module(tests, attr, globs, setUp, tearDown)
        else:
            module.__test__[attr.__name__] = attr

    tests.addTests(doctest.DocTestSuite(
        module,
        globs=globs,
        setUp=setUp,
        tearDown=tearDown,
        optionflags=doctest.ELLIPSIS,
    ))

    return tests


def load_tests(loader, tests, ignore):
    """load_test function used by unittest to find the doctests"""

    globs = {
        "fastobo": fastobo,
        "datetime": datetime,
        "pprint": pprint.pprint,
        "ms": fastobo.load(os.path.realpath(
            os.path.join(__file__, "..", "data", "ms.obo")
        )),
    }

    if not sys.argv[0].endswith('green'):
        tests = _load_tests_from_module(tests, fastobo, globs)
    return tests


def setUpModule():
    warnings.simplefilter('ignore')


def tearDownModule():
    warnings.simplefilter(warnings.defaultaction)



# coding: utf-8

import datetime
import unittest

import fastobo


class TestXref(unittest.TestCase):

    type = fastobo.xref.Xref

    def test_init(self):
        id = fastobo.id.PrefixedIdent('ISBN', '0321842685')
        try:
            xref = self.type(id)
        except Exception:
            self.fail("could not create `Xref` instance without description")
        try:
            xref = self.type(id, "Hacker's Delight (2nd Edition)")
        except Exception:
            self.fail("could not create `Xref` instance with description")

    def test_init_type_error(self):
        id = fastobo.id.PrefixedIdent('ISBN', '0321842685')
        desc = "Hacker's Delight (2nd Edition)"
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, 1, desc)
        self.assertRaises(TypeError, self.type, id, 1)

    def test_str(self):
        id = fastobo.id.PrefixedIdent('ISBN', '0321842685')
        desc = "Hacker's Delight (2nd Edition)"
        self.assertEqual(str(self.type(id)), "ISBN:0321842685")
        self.assertEqual(
            str(self.type(id, desc)),
            'ISBN:0321842685 "Hacker\'s Delight (2nd Edition)"'
        )


class TestXrefList(unittest.TestCase):

    type = fastobo.xref.XrefList

    def setUp(self):
        id = fastobo.id.PrefixedIdent('ISBN', '0321842685')
        desc = "Hacker's Delight (2nd Edition)"
        self.x1 = fastobo.xref.Xref(id, desc)
        self.x2 = fastobo.xref.Xref(fastobo.id.UnprefixedIdent("fastobo"))

    def test_init(self):
        try:
            xref = self.type()
        except Exception:
            self.fail("could not create `XrefList` instance without argument")
        try:
            xref = self.type([self.x1, self.x2])
        except Exception:
            self.fail("could not create `XrefList` instance from list")
        try:
            xref = self.type(iter([self.x1, self.x2]))
        except Exception:
            self.fail("could not create `XrefList` instance from iterator")

    def test_init_type_error(self):
        # Errors on an iterator of type != Xref
        self.assertRaises(TypeError, self.type, "abc")
        self.assertRaises(TypeError, self.type, ["abc", "def"])

    def test_str(self):
        x1, x2 = self.x1, self.x2
        self.assertEqual(str(self.type()), "[]")
        self.assertEqual(str(self.type([x1])), '[{}]'.format(x1))
        self.assertEqual(str(self.type([x1, x2])), '[{}, {}]'.format(x1, x2))

    def test_contains(self):
        x1, x2 = self.x1, self.x2
        l1 = self.type()
        self.assertNotIn(x1, l1)
        self.assertNotIn(x2, l1)
        l2 = self.type([x1])
        self.assertIn(x1, l2)
        self.assertNotIn(x2, l2)
        l3 = self.type([x1, x2])
        self.assertIn(x1, l3)
        self.assertIn(x2, l3)



# coding: utf-8

import datetime
import unittest

import fastobo

# --- HeaderFrame -----------------------------------------------------------

class TestHeaderFrame(unittest.TestCase):

    type = fastobo.header.HeaderFrame

    def test_init(self):
        try:
            frame = fastobo.header.HeaderFrame([])
        except Exception:
            self.fail("could not create `HeaderFrame` instance")
        try:
            frame = fastobo.header.HeaderFrame()
        except Exception:
            self.fail("could not create `HeaderFrame` instance")
        try:
            frame = fastobo.header.HeaderFrame((
                fastobo.header.FormatVersionClause("1.2"),
                fastobo.header.SavedByClause("Martin Larralde"),
            ))
        except Exception:
            self.fail("could not create `HeaderFrame` instance from iterable")

    def test_init_type_error(self):
        # self.assertRaises(TypeError, self.type, 1) # SEGFAULTS
        self.assertRaises(TypeError, self.type, [1])
        self.assertRaises(TypeError, self.type, ["abc"])
        self.assertRaises(TypeError, self.type, "abc")

# --- HeaderClause ----------------------------------------------------------

class _TestUnquotedStringClause(object):

    type = NotImplemented

    def test_init(self):
        try:
            vc = self.type("1.2")
        except Exception:
            self.fail("could not create `{}` instance", self.type.__name__)

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, [])

    def test_repr(self):
        x = self.type("abc")
        self.assertEqual(repr(x), "{}('abc')".format(self.type.__name__))

    def test_eq(self):
        x = self.type("1.2")
        self.assertEqual(x, self.type("1.2"))
        self.assertNotEqual(x, self.type("1.3"))

# --- FormatVersion ----------------------------------------------------------

class TestFormatVersionClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.FormatVersionClause

    def test_str(self):
        vc = fastobo.header.FormatVersionClause("1.2")
        self.assertEqual(str(vc), "format-version: 1.2")
        vc = fastobo.header.FormatVersionClause("x:y")
        self.assertEqual(str(vc), "format-version: x:y")

    def test_property_version(self):
        vc1 = self.type("1.2")
        self.assertEqual(vc1.version, "1.2")
        vc1.version = "1.3"
        self.assertEqual(vc1.version, "1.3")
        self.assertEqual(repr(vc1), "FormatVersionClause('1.3')")

# --- DataVersion ------------------------------------------------------------

class TestDataVersionClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.DataVersionClause

    def test_str(self):
        x = self.type("4.0")
        self.assertEqual(str(x), "data-version: 4.0")

    def test_property_version(self):
        vc1 = self.type("1.2")
        self.assertEqual(vc1.version, "1.2")
        vc1.version = "1.3"
        self.assertEqual(vc1.version, "1.3")
        self.assertEqual(repr(vc1), "DataVersionClause('1.3')")

# --- Date -------------------------------------------------------------------

class TestDateClause(unittest.TestCase):

    type = fastobo.header.DateClause

    def test_init(self):
        try:
            vc = self.type(datetime.datetime.now())
        except Exception:
            self.fail("could not create `{}` instance", self.type.__name__)

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, [])

    @unittest.expectedFailure
    def test_repr(self):
        now = datetime.datetime.now()
        x = self.type(now)
        self.assertEqual(repr(x), "{}({!r})".format(self.type.__name__, now))

    @unittest.expectedFailure
    def test_eq(self):
        now = datetime.datetime.now()
        x = self.type(now)
        self.assertEqual(x, self.type(now))
        self.assertNotEqual(x, self.type(datetime.datetime.now()))

    def test_str(self):
        then = datetime.datetime(2019, 4, 8, 16, 51)
        vc = self.type(then)
        self.assertEqual(str(vc), "date: 08:04:2019 16:51")

    @unittest.expectedFailure
    def test_property_version(self):
        now = datetime.datetime.now()
        vc1 = self.type(now)
        self.assertEqual(vc1.date, now)

        then = datetime.datetime(2019, 4, 8, 16, 51)
        vc1.date = then
        self.assertEqual(vc1.date, then)

        with self.assertRaises(TypeError):
            vc1.date = 1

# --- SavedBy ----------------------------------------------------------------

class TestSavedByClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.SavedByClause

# --- AutoGeneratedBy --------------------------------------------------------

class TestAutoGeneratedByClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.AutoGeneratedByClause

# --- Import -----------------------------------------------------------------
# --- Subsetdef --------------------------------------------------------------
# --- SynonymTypedef ---------------------------------------------------------
# --- DefaultNamespace -------------------------------------------------------
# --- IdspaceClause ----------------------------------------------------------
# --- TreatXrefsAsEquivalentClause -------------------------------------------
# --- TreatXrefsAsGenusDifferentiaClause -------------------------------------
# --- TreatXrefsAsReverseGenusDifferentiaClause ------------------------------
# --- TreatXrefsAsRelationshipClause -----------------------------------------
# --- TreatXrefsAsIsA --------------------------------------------------------
# --- TreatXrefsAsHasSubclassClause ------------------------------------------
# --- PropertyValue ----------------------------------------------------------
# --- Remark -----------------------------------------------------------------

class TestRemarkClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.RemarkClause

# --- Ontology ---------------------------------------------------------------

class TestOntologyClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.OntologyClause

# --- OwlAxioms --------------------------------------------------------------

class TestOwlAxiomsClause(_TestUnquotedStringClause, unittest.TestCase):

    type = fastobo.header.OwlAxiomsClause

# --- UnreservedClause -------------------------------------------------------



# coding: utf-8

import datetime
import unittest

import fastobo


class _TestBaseIdent(object):

    type = NotImplemented

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 123)
        self.assertRaises(TypeError, self.type, [])


class TestUnprefixedIdent(_TestBaseIdent, unittest.TestCase):

    type = fastobo.id.UnprefixedIdent

    def test_init(self):
        try:
            self.type('created_by')
        except Exception:
            self.fail("could not instantiate `UnprefixedIdent`")

    def test_eq(self):
        ident = self.type('derived_from')
        self.assertEqual(ident, self.type('derived_from'))
        self.assertNotEqual(ident, self.type('has_elements_from'))
        self.assertNotEqual(ident, 123)

    def test_cmp(self):
        ident = self.type('derived_from')
        self.assertLessEqual(ident, self.type('derived_from'))
        self.assertLessEqual(ident, self.type('has_elements_from'))
        self.assertRaises(TypeError, ident.__lt__, 123)
        self.assertRaises(TypeError, ident.__le__, 123)
        self.assertRaises(TypeError, ident.__gt__, 123)
        self.assertRaises(TypeError, ident.__ge__, 123)


class TestPrefixedIdent(_TestBaseIdent, unittest.TestCase):

    type = fastobo.id.PrefixedIdent

    def test_init(self):
        try:
            self.type('GO', '0070412')
        except Exception:
            self.fail("could not instantiate `PrefixedIdent`")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, "GO", 123)
        self.assertRaises(TypeError, self.type, "GO", [])
        self.assertRaises(TypeError, self.type, 123, "0070412")
        self.assertRaises(TypeError, self.type, [], "0070412")


class TestUrl(_TestBaseIdent, unittest.TestCase):

    type = fastobo.id.Url

    def test_init(self):
        try:
            self.type('http://purl.obolibrary.org/obo/GO_0070412')
        except Exception:
            self.fail("could not instantiate `Url`")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type, 123)
        self.assertRaises(TypeError, self.type, [])

    def test_init_value_error(self):
        self.assertRaises(ValueError, self.type, "not a URL at all")

    def test_eq(self):
        url = self.type('http://purl.obolibrary.org/obo/GO_0070412')
        self.assertEqual(url, self.type('http://purl.obolibrary.org/obo/GO_0070412'))
        self.assertNotEqual(url, self.type('http://purl.obolibrary.org/obo/GO_0070413'))
        self.assertNotEqual(url, 123)

    def test_cmp(self):
        url = self.type('http://purl.obolibrary.org/obo/GO_0070412')
        self.assertLess(url, self.type('http://purl.obolibrary.org/obo/GO_0070413'))
        self.assertRaises(TypeError, url.__lt__, 'http://purl.obolibrary.org/obo/GO_0070413')



#!/usr/bin/env python3

import os

import setuptools
import setuptools_rust as rust

setuptools.setup(
    setup_requires=["setuptools", "setuptools_rust"],
    rust_extensions=[rust.RustExtension(
        "fastobo",
        path="Cargo.toml",
        binding=rust.Binding.PyO3,
        strip=rust.Strip.Debug
    )],
)
